@@ -20,7 +20,6 @@ package coremain
 import (
 	"github.com/IrineSistiana/mosdns/v2/dispatcher/handler"
 	"gopkg.in/yaml.v3"
-	"io/ioutil"
 	"os"
 )
 
@@ -37,7 +36,12 @@ type Config struct {
 
 // parseConfig loads a yaml config from path f.
 func parseConfig(f string) (*Config, error) {
-	b, err := ioutil.ReadFile(f)
+	b, err := os.ReadFile(f)
+	if err != nil {
+		return nil, err
+	}
+
+	b, err = findAndReplaceCmd(b, 0, runCmd)
 	if err != nil {
 		return nil, err
 	}
