@@ -20,9 +20,9 @@ package ecs
 import (
 	"context"
 	"fmt"
-	"github.com/IrineSistiana/mosdns/dispatcher/handler"
-	"github.com/IrineSistiana/mosdns/dispatcher/pkg/dnsutils"
-	"github.com/IrineSistiana/mosdns/dispatcher/pkg/utils"
+	"github.com/IrineSistiana/mosdns/v2/dispatcher/handler"
+	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/dnsutils"
+	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/utils"
 	"github.com/miekg/dns"
 	"go.uber.org/zap"
 	"net"
@@ -104,9 +104,18 @@ func newPlugin(bp *handler.BP, args *Args) (p handler.Plugin, err error) {
 }
 
 // Exec tries to append ECS to qCtx.Q().
-// If an error occurred, Do will just log it.
-// Therefore, Do will never return an err.
-func (e ecsPlugin) Exec(_ context.Context, qCtx *handler.Context) (_ error) {
+// If an error occurred, Exec will just log it to internal logger.
+// It will never raise its own error.
+func (e ecsPlugin) Exec(ctx context.Context, qCtx *handler.Context, next handler.ExecutableChainNode) error {
+	err := e.appendECS(qCtx)
+	if err != nil {
+		e.L().Warn("internal err", zap.Error(err))
+	}
+
+	return handler.ExecChainNode(ctx, qCtx, next)
+}
+
+func (e ecsPlugin) appendECS(qCtx *handler.Context) error {
 	qHasECS := dnsutils.GetMsgECS(qCtx.Q()) != nil
 	if qHasECS && !e.args.ForceOverwrite {
 		return nil
@@ -116,8 +125,7 @@ func (e ecsPlugin) Exec(_ context.Context, qCtx *handler.Context) (_ error) {
 	if e.args.Auto && qCtx.From() != nil { // use client ip
 		ip := utils.GetIPFromAddr(qCtx.From())
 		if ip == nil {
-			e.L().Warn("internal err: can not parse client ip address", qCtx.InfoField(), zap.Stringer("from", qCtx.From()))
-			return nil
+			return fmt.Errorf("failed to parse client ip address, the raw data is [%s]", qCtx.From())
 		}
 		if ip4 := ip.To4(); ip4 != nil { // is ipv4
 			ecs = dnsutils.NewEDNS0Subnet(ip4, e.args.Mask4, false)
@@ -125,8 +133,7 @@ func (e ecsPlugin) Exec(_ context.Context, qCtx *handler.Context) (_ error) {
 			if ip6 := ip.To16(); ip6 != nil { // is ipv6
 				ecs = dnsutils.NewEDNS0Subnet(ip6, e.args.Mask6, true)
 			} else { // non
-				e.L().Warn("internal err: client ip address is not a valid ip address", qCtx.InfoField(), zap.Stringer("from", qCtx.From()))
-				return nil
+				return fmt.Errorf("invalid client ip address [%s]", qCtx.From())
 			}
 		}
 	} else { // use preset ip
@@ -149,17 +156,6 @@ func (e ecsPlugin) Exec(_ context.Context, qCtx *handler.Context) (_ error) {
 
 	if ecs != nil {
 		dnsutils.AppendECS(qCtx.Q(), ecs)
-
-		// According to https://tools.ietf.org/html/rfc7871#section-7.2.2
-		// > Because a client that did not use an ECS option might not
-		// > be able to understand it, the server MUST NOT provide one in
-		// > its response.
-		//
-		// If the original query did not have an ECS option, remove ECS from
-		// its response.
-		if !qHasECS {
-			qCtx.DeferExec(removeResponseECS{})
-		}
 	}
 
 	return nil
@@ -178,20 +174,10 @@ type noECS struct {
 
 var _ handler.ExecutablePlugin = (*noECS)(nil)
 
-func (n noECS) Exec(_ context.Context, qCtx *handler.Context) (_ error) {
+func (n noECS) Exec(ctx context.Context, qCtx *handler.Context, next handler.ExecutableChainNode) error {
 	dnsutils.RemoveECS(qCtx.Q())
 	if qCtx.R() != nil {
 		dnsutils.RemoveECS(qCtx.R())
 	}
-	return
-}
-
-type removeResponseECS struct{}
-
-func (e removeResponseECS) Exec(_ context.Context, qCtx *handler.Context) (_ error) {
-	r := qCtx.R()
-	if r != nil {
-		dnsutils.RemoveECS(r)
-	}
-	return nil
+	return handler.ExecChainNode(ctx, qCtx, next)
 }
